@@ -37,11 +37,11 @@
 #include "photos-main-toolbar.h"
 #include "photos-mode-controller.h"
 #include "photos-notification-manager.h"
-#include "photos-offset-controller.h"
+#include "photos-offset-overview-controller.h"
 #include "photos-selection-toolbar.h"
 #include "photos-spinner-box.h"
 #include "photos-tracker-change-monitor.h"
-#include "photos-tracker-controller.h"
+#include "photos-tracker-overview-controller.h"
 #include "photos-view-container.h"
 
 
@@ -50,7 +50,9 @@ struct _PhotosEmbedPrivate
   ClutterActor *background;
   ClutterActor *contents_actor;
   ClutterActor *error_box;
+  ClutterActor *favorites_actor;
   ClutterActor *image_actor;
+  ClutterActor *image_container;
   ClutterActor *no_results;
   ClutterActor *ntfctn_mngr;
   ClutterActor *overview_actor;
@@ -59,6 +61,7 @@ struct _PhotosEmbedPrivate
   ClutterLayoutManager *contents_layout;
   ClutterLayoutManager *view_layout;
   GCancellable *loader_cancellable;
+  GtkWidget *favorites;
   GtkWidget *indexing_ntfctn;
   GtkWidget *overview;
   PhotosBaseManager *item_mngr;
@@ -67,7 +70,7 @@ struct _PhotosEmbedPrivate
   PhotosModeController *mode_cntrlr;
   PhotosOffsetController *offset_cntrlr;
   PhotosTrackerChangeMonitor *monitor;
-  PhotosTrackerController *trk_cntrlr;
+  PhotosTrackerController *trk_ovrvw_cntrlr;
   gint preview_page;
   gint view_page;
   gulong no_results_change_id;
@@ -77,6 +80,41 @@ struct _PhotosEmbedPrivate
 G_DEFINE_TYPE (PhotosEmbed, photos_embed, PHOTOS_TYPE_EMBED_WIDGET);
 
 
+static void
+photos_embed_scale_image (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+  ClutterContent *content;
+  gfloat height_a;
+  gfloat height_c;
+  gfloat width_a;
+  gfloat width_c;
+
+  content = clutter_actor_get_content (priv->image_actor);
+  if (content == NULL)
+    return;
+
+  if (!clutter_content_get_preferred_size (content, &width_c, &height_c))
+    return;
+
+  clutter_actor_get_size (priv->image_container, &width_a, &height_a);
+  if (width_c < width_a && height_c < height_a)
+    {
+      clutter_actor_set_content_gravity (priv->image_actor, CLUTTER_CONTENT_GRAVITY_CENTER);
+      clutter_actor_set_size (priv->image_actor, width_c, height_c);
+      clutter_actor_set_x_expand (priv->image_actor, FALSE);
+      clutter_actor_set_y_expand (priv->image_actor, FALSE);
+    }
+  else
+    {
+      clutter_actor_set_content_gravity (priv->image_actor, CLUTTER_CONTENT_GRAVITY_RESIZE_ASPECT);
+      clutter_actor_set_size (priv->image_actor, -1.0, -1.0);
+      clutter_actor_set_x_expand (priv->image_actor, TRUE);
+      clutter_actor_set_y_expand (priv->image_actor, TRUE);
+    }
+}
+
+
 static void
 photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -109,12 +147,11 @@ photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user
   clutter_image_set_data (CLUTTER_IMAGE (image), pixels, pixel_format, width, height, row_stride, NULL);
   g_object_unref (pixbuf);
 
-  clutter_actor_save_easing_state (priv->image_actor);
-  clutter_actor_set_content_gravity (priv->image_actor, CLUTTER_CONTENT_GRAVITY_RESIZE_ASPECT);
-  clutter_actor_restore_easing_state (priv->image_actor);
   clutter_actor_set_content (priv->image_actor, image);
   g_object_unref (image);
 
+  photos_embed_scale_image (self);
+
   /* TODO: set toolbar model */
 
   photos_spinner_box_move_out (PHOTOS_SPINNER_BOX (priv->spinner_box));
@@ -156,6 +193,7 @@ photos_embed_hide_no_results_page (PhotosEmbed *self)
     }
 
   clutter_actor_set_child_below_sibling (priv->view_actor, priv->no_results, NULL);
+  clutter_actor_hide (priv->no_results);
 }
 
 
@@ -177,6 +215,7 @@ photos_embed_count_changed (PhotosEmbed *self, gint count)
                                                              "changes-pending",
                                                              G_CALLBACK (photos_embed_changes_pending),
                                                              self);
+      clutter_actor_show (priv->no_results);
       clutter_actor_set_child_above_sibling (priv->view_actor, priv->no_results, NULL);
     }
   else
@@ -190,6 +229,27 @@ photos_embed_fullscreen_changed (PhotosModeController *mode_cntrlr, gboolean ful
 }
 
 
+static void
+photos_embed_prepare_for_favorites (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+
+  photos_base_manager_set_active_object (priv->item_mngr, NULL);
+
+  if (priv->loader_cancellable != NULL)
+    {
+      g_cancellable_cancel (priv->loader_cancellable);
+      g_clear_object (&priv->loader_cancellable);
+    }
+
+  photos_spinner_box_move_out (PHOTOS_SPINNER_BOX (priv->spinner_box));
+  photos_error_box_move_out (PHOTOS_ERROR_BOX (priv->error_box));
+
+  clutter_actor_show (priv->favorites_actor);
+  clutter_actor_set_child_below_sibling (priv->view_actor, priv->favorites_actor, priv->ntfctn_mngr);
+}
+
+
 static void
 photos_embed_prepare_for_overview (PhotosEmbed *self)
 {
@@ -206,6 +266,7 @@ photos_embed_prepare_for_overview (PhotosEmbed *self)
   photos_spinner_box_move_out (PHOTOS_SPINNER_BOX (priv->spinner_box));
   photos_error_box_move_out (PHOTOS_ERROR_BOX (priv->error_box));
 
+  clutter_actor_show (priv->overview_actor);
   clutter_actor_set_child_below_sibling (priv->view_actor, priv->overview_actor, priv->ntfctn_mngr);
 }
 
@@ -219,7 +280,8 @@ photos_embed_prepare_for_preview (PhotosEmbed *self)
    *       ErrorHandler
    */
 
-  clutter_actor_set_child_below_sibling (priv->view_actor, priv->image_actor, priv->ntfctn_mngr);
+  clutter_actor_show (priv->image_container);
+  clutter_actor_set_child_below_sibling (priv->view_actor, priv->image_container, priv->ntfctn_mngr);
 }
 
 
@@ -255,32 +317,6 @@ photos_embed_query_status_changed (PhotosTrackerController *trk_cntrlr, gboolean
     photos_spinner_box_move_out (PHOTOS_SPINNER_BOX (priv->spinner_box));
 }
 
-
-static void
-photos_embed_window_mode_change_flash_completed (PhotosEmbed *self)
-{
-  PhotosEmbedPrivate *priv = self->priv;
-  clutter_actor_set_child_below_sibling (priv->view_actor, priv->background, NULL);
-}
-
-
-static void
-photos_embed_window_mode_change_flash (PhotosEmbed *self)
-{
-  PhotosEmbedPrivate *priv = self->priv;
-  ClutterAnimation *animation;
-
-  clutter_actor_set_child_above_sibling (priv->view_actor, priv->background, NULL);
-  clutter_actor_set_opacity (priv->background, 255);
-
-  animation = clutter_actor_animate (priv->background, CLUTTER_EASE_IN_QUAD, 200, "opacity", 0, NULL);
-  g_signal_connect_swapped (animation,
-                            "completed",
-                            G_CALLBACK (photos_embed_window_mode_change_flash_completed),
-                            self);
-}
-
-
 static void
 photos_embed_window_mode_changed (PhotosModeController *mode_cntrlr,
                                   PhotosWindowMode mode,
@@ -288,14 +324,18 @@ photos_embed_window_mode_changed (PhotosModeController *mode_cntrlr,
                                   gpointer user_data)
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
+  PhotosEmbedPrivate *priv = self->priv;
 
-  if (mode == PHOTOS_WINDOW_MODE_OVERVIEW)
+  clutter_actor_hide (priv->favorites_actor);
+  clutter_actor_hide (priv->image_container);
+  clutter_actor_hide (priv->overview_actor);
+
+  if (mode == PHOTOS_WINDOW_MODE_FAVORITES)
+    photos_embed_prepare_for_favorites (self);
+  else if (mode == PHOTOS_WINDOW_MODE_OVERVIEW)
     photos_embed_prepare_for_overview (self);
   else
     photos_embed_prepare_for_preview (self);
-
-  if (old_mode != PHOTOS_WINDOW_MODE_NONE)
-    photos_embed_window_mode_change_flash (self);
 }
 
 
@@ -315,20 +355,52 @@ photos_embed_dispose (GObject *object)
   g_clear_object (&priv->loader_cancellable);
   g_clear_object (&priv->indexing_ntfctn);
   g_clear_object (&priv->item_mngr);
-
-  if (priv->mode_cntrlr != NULL)
-    {
-      g_object_unref (priv->mode_cntrlr);
-      priv->mode_cntrlr = NULL;
-    }
-
+  g_clear_object (&priv->mode_cntrlr);
   g_clear_object (&priv->offset_cntrlr);
   g_clear_object (&priv->monitor);
-  g_clear_object (&priv->trk_cntrlr);
+  g_clear_object (&priv->trk_ovrvw_cntrlr);
 
   G_OBJECT_CLASS (photos_embed_parent_class)->dispose (object);
 }
 
+static gboolean
+on_image_background_draw (ClutterCanvas *canvas,
+                          cairo_t       *cr,
+                          gint           width,
+                          gint           height,
+                          gpointer       user_data)
+{
+  PhotosEmbed *self = PHOTOS_EMBED (user_data);
+  GtkStyleContext *context;
+  GtkStateFlags flags;
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (self));
+  flags = gtk_widget_get_state_flags (GTK_WIDGET (self));
+  gtk_style_context_save (context);
+  gtk_style_context_set_state (context, flags);
+  gtk_render_background (context, cr, 0, 0, width, height);
+  gtk_style_context_restore (context);
+
+  return TRUE;
+}
+
+static void
+photos_embed_image_container_allocation_changed (ClutterActor           *actor,
+                                                 const ClutterActorBox  *allocation,
+                                                 ClutterAllocationFlags  flags,
+                                                 gpointer                user_data)
+{
+  PhotosEmbed *self = PHOTOS_EMBED (user_data);
+  ClutterContent *content;
+
+  content = clutter_actor_get_content (actor);
+  clutter_canvas_set_size (CLUTTER_CANVAS (content),
+                           allocation->x2 - allocation->x1,
+                           allocation->y2 - allocation->y1);
+
+  photos_embed_scale_image (self);
+}
+
 
 static void
 photos_embed_init (PhotosEmbed *self)
@@ -337,14 +409,20 @@ photos_embed_init (PhotosEmbed *self)
   ClutterActor *actor;
   ClutterActor *stage;
   ClutterActor *toolbar_actor;
+  ClutterLayoutManager *image_layout;
   ClutterLayoutManager *overlay_layout;
   ClutterConstraint *constraint;
+  ClutterContent *image_background;
   gboolean querying;
+  GtkStyleContext *context;
 
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_EMBED, PhotosEmbedPrivate);
   priv = self->priv;
 
   gtk_clutter_embed_set_use_layout_size (GTK_CLUTTER_EMBED (self), TRUE);
+  context = gtk_widget_get_style_context (GTK_WIDGET (self));
+  gtk_style_context_add_class (context, GTK_STYLE_CLASS_VIEW);
+  gtk_style_context_add_class (context, "content-view");
 
   overlay_layout = clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_CENTER, CLUTTER_BIN_ALIGNMENT_CENTER);
   actor = clutter_actor_new ();
@@ -380,20 +458,40 @@ photos_embed_init (PhotosEmbed *self)
 
   priv->indexing_ntfctn = g_object_ref_sink (photos_indexing_notification_new ());
 
-  priv->overview = photos_view_container_new ();
+  priv->overview = photos_view_container_new (PHOTOS_WINDOW_MODE_OVERVIEW);
   priv->overview_actor = gtk_clutter_actor_new_with_contents (priv->overview);
   clutter_actor_set_x_expand (priv->overview_actor, TRUE);
   clutter_actor_set_y_expand (priv->overview_actor, TRUE);
   clutter_actor_insert_child_below (priv->view_actor, priv->overview_actor, NULL);
 
+  priv->favorites = photos_view_container_new (PHOTOS_WINDOW_MODE_FAVORITES);
+  priv->favorites_actor = gtk_clutter_actor_new_with_contents (priv->favorites);
+  clutter_actor_set_x_expand (priv->favorites_actor, TRUE);
+  clutter_actor_set_y_expand (priv->favorites_actor, TRUE);
+  clutter_actor_insert_child_below (priv->view_actor, priv->favorites_actor, NULL);
+
+  priv->image_container = clutter_actor_new ();
+  clutter_actor_set_x_expand (priv->image_container, TRUE);
+  clutter_actor_set_y_expand (priv->image_container, TRUE);
+  image_layout = clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_CENTER, CLUTTER_BIN_ALIGNMENT_CENTER);
+  clutter_actor_set_layout_manager (priv->image_container, image_layout);
+  clutter_actor_insert_child_below (priv->view_actor, priv->image_container, NULL);
+
+  image_background = clutter_canvas_new ();
+  clutter_actor_set_content (priv->image_container, image_background);
+  g_signal_connect (priv->image_container, "allocation-changed",
+                    G_CALLBACK (photos_embed_image_container_allocation_changed), self);
+  g_signal_connect (image_background, "draw",
+                    G_CALLBACK (on_image_background_draw), self);
+  g_signal_connect_swapped (self, "state-flags-changed",
+                            G_CALLBACK (clutter_content_invalidate), image_background);
+  g_object_unref (image_background);
+
   priv->image_actor = clutter_actor_new ();
-  clutter_actor_set_background_color (priv->image_actor, CLUTTER_COLOR_Black);
   clutter_actor_set_content_scaling_filters (priv->image_actor,
                                              CLUTTER_SCALING_FILTER_TRILINEAR,
                                              CLUTTER_SCALING_FILTER_TRILINEAR);
-  clutter_actor_set_x_expand (priv->image_actor, TRUE);
-  clutter_actor_set_y_expand (priv->image_actor, TRUE);
-  clutter_actor_insert_child_below (priv->view_actor, priv->image_actor, NULL);
+  clutter_actor_add_child (priv->image_container, priv->image_actor);
 
   priv->spinner_box = photos_spinner_box_new ();
   clutter_actor_insert_child_below (priv->view_actor, priv->spinner_box, NULL);
@@ -405,7 +503,7 @@ photos_embed_init (PhotosEmbed *self)
   clutter_actor_insert_child_below (priv->view_actor, priv->no_results, NULL);
 
   priv->background = clutter_actor_new ();
-  clutter_actor_set_background_color (priv->background, CLUTTER_COLOR_White);
+  clutter_actor_set_background_color (priv->background, CLUTTER_COLOR_Black);
   clutter_actor_set_x_expand (priv->background, TRUE);
   clutter_actor_set_y_expand (priv->background, TRUE);
   clutter_actor_insert_child_below (priv->view_actor, priv->background, NULL);
@@ -428,21 +526,21 @@ photos_embed_init (PhotosEmbed *self)
                     G_CALLBACK (photos_embed_fullscreen_changed),
                     self);
 
-  priv->trk_cntrlr = photos_tracker_controller_new ();
-  g_signal_connect_swapped (priv->trk_cntrlr, "query-error", G_CALLBACK (photos_embed_query_error), self);
-  g_signal_connect (priv->trk_cntrlr,
+  priv->trk_ovrvw_cntrlr = photos_tracker_overview_controller_new ();
+  g_signal_connect_swapped (priv->trk_ovrvw_cntrlr, "query-error", G_CALLBACK (photos_embed_query_error), self);
+  g_signal_connect (priv->trk_ovrvw_cntrlr,
                     "query-status-changed",
                     G_CALLBACK (photos_embed_query_status_changed),
                     self);
 
-  priv->offset_cntrlr = photos_offset_controller_new ();
+  priv->offset_cntrlr = photos_offset_overview_controller_new ();
   g_signal_connect_swapped (priv->offset_cntrlr, "count-changed", G_CALLBACK (photos_embed_count_changed), self);
 
   priv->item_mngr = photos_item_manager_new ();
   g_signal_connect (priv->item_mngr, "active-changed", G_CALLBACK (photos_embed_active_changed), self);
 
-  querying = photos_tracker_controller_get_query_status (priv->trk_cntrlr);
-  photos_embed_query_status_changed (priv->trk_cntrlr, querying, self);
+  querying = photos_tracker_controller_get_query_status (priv->trk_ovrvw_cntrlr);
+  photos_embed_query_status_changed (priv->trk_ovrvw_cntrlr, querying, self);
 
   priv->monitor = photos_tracker_change_monitor_new ();
 
