@@ -50,6 +50,7 @@ struct _PhotosMainToolbarPrivate
   PhotosBaseManager *src_mngr;
   PhotosModeController *mode_cntrlr;
   PhotosSelectionController *sel_cntrlr;
+  PhotosWindowMode old_mode;
   gulong collection_id;
   gulong search_source_id;
   gulong selection_changed_id;
@@ -61,6 +62,44 @@ struct _PhotosMainToolbarPrivate
 G_DEFINE_TYPE (PhotosMainToolbar, photos_main_toolbar, G_TYPE_OBJECT);
 
 
+static void
+photos_main_toolbar_favorites_toggled (GtkToggleButton *toggle_button, gpointer user_data)
+{
+  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
+
+  if (gtk_toggle_button_get_active (toggle_button))
+    photos_mode_controller_set_window_mode (self->priv->mode_cntrlr, PHOTOS_WINDOW_MODE_FAVORITES);
+}
+
+
+static void
+photos_main_toolbar_overview_toggled (GtkToggleButton *toggle_button, gpointer user_data)
+{
+  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
+
+  if (gtk_toggle_button_get_active (toggle_button))
+    photos_mode_controller_set_window_mode (self->priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
+}
+
+
+static void
+photos_main_toolbar_add_modes (PhotosMainToolbar *self, PhotosWindowMode window_mode)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  GtkWidget *button;
+
+  button = gd_main_toolbar_add_mode (GD_MAIN_TOOLBAR (priv->widget), _("Photos"));
+  if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+  g_signal_connect (button, "toggled", G_CALLBACK (photos_main_toolbar_overview_toggled), self);
+
+  button = gd_main_toolbar_add_mode (GD_MAIN_TOOLBAR (priv->widget), _("Favorites"));
+  if (window_mode == PHOTOS_WINDOW_MODE_FAVORITES)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+  g_signal_connect (button, "toggled", G_CALLBACK (photos_main_toolbar_favorites_toggled), self);
+}
+
+
 static void
 photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
 {
@@ -175,7 +214,7 @@ photos_main_toolbar_back_button_clicked (GtkButton *button, gpointer user_data)
   PhotosMainToolbarPrivate *priv = self->priv;
 
   photos_base_manager_set_active_object (priv->item_mngr, NULL);
-  photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
+  photos_mode_controller_set_window_mode (priv->mode_cntrlr, priv->old_mode);
 }
 
 
@@ -256,6 +295,32 @@ photos_main_toolbar_select_button_clicked (GtkButton *button, gpointer user_data
 }
 
 
+static void
+photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  GObject *object;
+  GtkWidget *selection_button;
+
+  gd_main_toolbar_set_show_modes (GD_MAIN_TOOLBAR (priv->widget), TRUE);
+  photos_main_toolbar_add_modes (self, PHOTOS_WINDOW_MODE_FAVORITES);
+
+  selection_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->widget),
+                                                 "object-select-symbolic",
+                                                 _("Select Items"),
+                                                 FALSE);
+  g_signal_connect (selection_button, "clicked", G_CALLBACK (photos_main_toolbar_select_button_clicked), self);
+
+  priv->collection_id = g_signal_connect (priv->col_mngr,
+                                          "active-changed",
+                                          G_CALLBACK (photos_main_toolbar_active_changed),
+                                          self);
+
+  object = photos_base_manager_get_active_object (priv->col_mngr);
+  photos_main_toolbar_active_changed (priv->col_mngr, object, self);
+}
+
+
 static void
 photos_main_toolbar_populate_for_overview (PhotosMainToolbar *self)
 {
@@ -263,6 +328,9 @@ photos_main_toolbar_populate_for_overview (PhotosMainToolbar *self)
   GObject *object;
   GtkWidget *selection_button;
 
+  gd_main_toolbar_set_show_modes (GD_MAIN_TOOLBAR (priv->widget), TRUE);
+  photos_main_toolbar_add_modes (self, PHOTOS_WINDOW_MODE_OVERVIEW);
+
   selection_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->widget),
                                                  "object-select-symbolic",
                                                  _("Select Items"),
@@ -287,6 +355,8 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   GtkWidget *back_button;
   const gchar *icon_name;
 
+  gd_main_toolbar_set_show_modes (GD_MAIN_TOOLBAR (priv->widget), FALSE);
+
   direction = gtk_widget_get_direction (GTK_WIDGET (priv->widget));
   icon_name = (direction == GTK_TEXT_DIR_RTL) ? "go-next-symbolic" : "go-previous-symbolic";
   back_button = gd_main_toolbar_add_button (GD_MAIN_TOOLBAR (priv->widget), icon_name, _("Back"), TRUE);
@@ -301,6 +371,8 @@ photos_main_toolbar_populate_for_selection_mode (PhotosMainToolbar *self)
   GtkStyleContext *context;
   GtkWidget *selection_button;
 
+  gd_main_toolbar_set_show_modes (GD_MAIN_TOOLBAR (priv->widget), FALSE);
+
   context = gtk_widget_get_style_context (priv->widget);
   gtk_style_context_add_class (context, "selection-mode");
   gtk_widget_reset_style (priv->widget);
@@ -322,20 +394,18 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv = self->priv;
   PhotosWindowMode window_mode;
+  gboolean selection_mode;
 
   photos_main_toolbar_clear_toolbar (self);
+  selection_mode = photos_selection_controller_get_selection_mode (priv->sel_cntrlr);
   window_mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
 
-  if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW)
-    {
-      gboolean selection_mode;
-
-      selection_mode = photos_selection_controller_get_selection_mode (priv->sel_cntrlr);
-      if (selection_mode)
-        photos_main_toolbar_populate_for_selection_mode (self);
-      else
-        photos_main_toolbar_populate_for_overview (self);
-    }
+  if (selection_mode)
+    photos_main_toolbar_populate_for_selection_mode (self);
+  else if (window_mode == PHOTOS_WINDOW_MODE_FAVORITES)
+    photos_main_toolbar_populate_for_favorites (self);
+  else if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW)
+    photos_main_toolbar_populate_for_overview (self);
   else if (window_mode == PHOTOS_WINDOW_MODE_PREVIEW)
     photos_main_toolbar_populate_for_preview (self);
 
@@ -344,6 +414,14 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
 }
 
 
+static void
+photos_main_toolbar_window_mode_changed (PhotosMainToolbar *self, PhotosWindowMode mode, PhotosWindowMode old_mode)
+{
+  self->priv->old_mode = old_mode;
+  photos_main_toolbar_reset_toolbar_mode (self);
+}
+
+
 static void
 photos_main_toolbar_dispose (GObject *object)
 {
@@ -353,18 +431,8 @@ photos_main_toolbar_dispose (GObject *object)
   g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
-
-  if (priv->mode_cntrlr != NULL)
-    {
-      g_object_unref (priv->mode_cntrlr);
-      priv->mode_cntrlr = NULL;
-    }
-
-  if (priv->sel_cntrlr != NULL)
-    {
-      g_object_unref (priv->sel_cntrlr);
-      priv->sel_cntrlr = NULL;
-    }
+  g_clear_object (&priv->mode_cntrlr);
+  g_clear_object (&priv->sel_cntrlr);
 
   G_OBJECT_CLASS (photos_main_toolbar_parent_class)->dispose (object);
 }
@@ -411,9 +479,10 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
                                                      self);
 
   priv->mode_cntrlr = photos_mode_controller_new ();
+  priv->old_mode = PHOTOS_WINDOW_MODE_NONE;
   priv->window_mode_id = g_signal_connect_swapped (priv->mode_cntrlr,
                                                    "window-mode-changed",
-                                                   G_CALLBACK (photos_main_toolbar_reset_toolbar_mode),
+                                                   G_CALLBACK (photos_main_toolbar_window_mode_changed),
                                                    self);
 
   priv->sel_cntrlr = photos_selection_controller_new ();
