@@ -27,7 +27,8 @@
 
 #include <gio/gio.h>
 
-#include "photos-offset-controller.h"
+#include "photos-offset-favorites-controller.h"
+#include "photos-offset-overview-controller.h"
 #include "photos-query-builder.h"
 #include "photos-source-manager.h"
 #include "photos-search-type-manager.h"
@@ -96,7 +97,10 @@ photos_query_builder_query (gboolean global, gint flags)
   gchar *where_sparql;
 
   optional = photos_query_builder_optional ();
-  where_sparql = g_strconcat ("WHERE { ?urn a rdfs:Resource ", optional, NULL);
+  if (flags & PHOTOS_QUERY_FLAGS_FAVORITES)
+    where_sparql = g_strconcat ("WHERE { ?urn nao:hasTag nao:predefined-tag-favorite ", optional, NULL);
+  else
+    where_sparql = g_strconcat ("WHERE { ?urn a rdfs:Resource ", optional, NULL);
   g_free (optional);
 
   if (!(flags & PHOTOS_QUERY_FLAGS_UNFILTERED))
@@ -123,7 +127,11 @@ photos_query_builder_query (gboolean global, gint flags)
       gint offset;
       gint step;
 
-      offset_cntrlr = photos_offset_controller_new ();
+      if (flags & PHOTOS_QUERY_FLAGS_FAVORITES)
+        offset_cntrlr = photos_offset_favorites_controller_new ();
+      else
+        offset_cntrlr = photos_offset_overview_controller_new ();
+
       offset = photos_offset_controller_get_offset (offset_cntrlr);
       step = photos_offset_controller_get_step (offset_cntrlr);
       g_object_unref (offset_cntrlr);
@@ -153,6 +161,27 @@ photos_query_builder_query (gboolean global, gint flags)
 }
 
 
+PhotosQuery *
+photos_query_builder_count_favorites_query (void)
+{
+  gchar *filter;
+  gchar *optional;
+  gchar *sparql;
+
+  filter = photos_query_builder_filter ();
+  optional = photos_query_builder_optional ();
+  sparql = g_strconcat ("SELECT DISTINCT COUNT(?urn) WHERE { ?urn nao:hasTag nao:predefined-tag-favorite ",
+                        optional,
+                        filter,
+                        " }", NULL);
+
+  g_free (optional);
+  g_free (filter);
+
+  return photos_query_new (sparql);
+}
+
+
 PhotosQuery *
 photos_query_builder_count_query (void)
 {
@@ -171,6 +200,16 @@ photos_query_builder_count_query (void)
 }
 
 
+PhotosQuery *
+photos_query_builder_global_favorites_query (void)
+{
+  gchar *sparql;
+
+  sparql = photos_query_builder_query (TRUE, PHOTOS_QUERY_FLAGS_FAVORITES);
+  return photos_query_new (sparql);
+}
+
+
 PhotosQuery *
 photos_query_builder_global_query (void)
 {
@@ -189,7 +228,7 @@ photos_query_builder_single_query (gint flags, const gchar *resource)
   gchar *sparql;
   gchar *tmp;
 
-  tmp = photos_query_builder_query (TRUE, flags);
+  tmp = photos_query_builder_query (FALSE, flags);
 
   regex = g_regex_new ("\\?urn", 0, 0, NULL);
   replacement = g_strconcat ("<", resource, ">", NULL);
