@@ -31,8 +31,12 @@
 
 G_BEGIN_DECLS
 
+PhotosQuery  *photos_query_builder_count_favorites_query (void);
+
 PhotosQuery  *photos_query_builder_count_query         (void);
 
+PhotosQuery  *photos_query_builder_global_favorites_query (void);
+
 PhotosQuery  *photos_query_builder_global_query        (void);
 
 PhotosQuery  *photos_query_builder_single_query        (gint flags, const gchar *resource);
