@@ -30,7 +30,6 @@
 
 #include "photos-item-manager.h"
 #include "photos-marshalers.h"
-#include "photos-offset-controller.h"
 #include "photos-query-builder.h"
 #include "photos-source-manager.h"
 #include "photos-tracker-controller.h"
@@ -62,7 +61,7 @@ enum
 static guint signals[LAST_SIGNAL] = { 0 };
 
 
-G_DEFINE_TYPE (PhotosTrackerController, photos_tracker_controller, G_TYPE_OBJECT);
+G_DEFINE_ABSTRACT_TYPE (PhotosTrackerController, photos_tracker_controller, G_TYPE_OBJECT);
 
 
 typedef enum
@@ -169,7 +168,7 @@ photos_tracker_controller_perform_current_query (PhotosTrackerController *self)
   if (priv->current_query != NULL)
     photos_query_free (priv->current_query);
 
-  priv->current_query = photos_query_builder_global_query ();
+  priv->current_query = PHOTOS_TRACKER_CONTROLLER_GET_CLASS (self)->get_query ();
   g_cancellable_reset (priv->cancellable);
 
   photos_tracker_queue_select (priv->queue,
@@ -254,23 +253,19 @@ photos_tracker_controller_source_object_removed (PhotosBaseManager *manager, GOb
 }
 
 
-static GObject *
-photos_tracker_controller_constructor (GType type,
-                                       guint n_construct_params,
-                                       GObjectConstructParam *construct_params)
+static void
+photos_tracker_controller_constructed (GObject *object)
 {
-  static GObject *self = NULL;
+  PhotosTrackerController *self = PHOTOS_TRACKER_CONTROLLER (object);
+  PhotosTrackerControllerPrivate *priv = self->priv;
 
-  if (self == NULL)
-    {
-      self = G_OBJECT_CLASS (photos_tracker_controller_parent_class)->constructor (type,
-                                                                                   n_construct_params,
-                                                                                   construct_params);
-      g_object_add_weak_pointer (self, (gpointer) &self);
-      return self;
-    }
+  G_OBJECT_CLASS (photos_tracker_controller_parent_class)->constructed (object);
 
-  return g_object_ref (self);
+  priv->offset_cntrlr = PHOTOS_TRACKER_CONTROLLER_GET_CLASS (self)->get_offset_controller ();
+  g_signal_connect (priv->offset_cntrlr,
+                    "offset-changed",
+                    G_CALLBACK (photos_tracker_controller_offset_changed),
+                    self);
 }
 
 
@@ -327,12 +322,6 @@ photos_tracker_controller_init (PhotosTrackerController *self)
                     G_CALLBACK (photos_tracker_controller_refresh_for_object),
                     self);
 
-  priv->offset_cntrlr = photos_offset_controller_new ();
-  g_signal_connect (priv->offset_cntrlr,
-                    "offset-changed",
-                    G_CALLBACK (photos_tracker_controller_offset_changed),
-                    self);
-
   priv->queue = photos_tracker_queue_new ();
 }
 
@@ -342,7 +331,7 @@ photos_tracker_controller_class_init (PhotosTrackerControllerClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
-  object_class->constructor = photos_tracker_controller_constructor;
+  object_class->constructed = photos_tracker_controller_constructed;
   object_class->dispose = photos_tracker_controller_dispose;
   object_class->finalize = photos_tracker_controller_finalize;
 
@@ -375,13 +364,6 @@ photos_tracker_controller_class_init (PhotosTrackerControllerClass *class)
 }
 
 
-PhotosTrackerController *
-photos_tracker_controller_new (void)
-{
-  return g_object_new (PHOTOS_TYPE_TRACKER_CONTROLLER, NULL);
-}
-
-
 void
 photos_tracker_controller_start (PhotosTrackerController *self)
 {
