@@ -27,6 +27,9 @@
 
 #include <glib-object.h>
 
+#include "photos-offset-controller.h"
+#include "photos-query.h"
+
 G_BEGIN_DECLS
 
 #define PHOTOS_TYPE_TRACKER_CONTROLLER (photos_tracker_controller_get_type ())
@@ -65,14 +68,17 @@ struct _PhotosTrackerControllerClass
 {
   GObjectClass parent_class;
 
+  /* virtual methods */
+  PhotosOffsetController *(*get_offset_controller) (void);
+  PhotosQuery *(*get_query) (void);
+
+  /* signals */
   void (*query_error) (PhotosTrackerController *self, const gchar *primary, const gchar *secondary);
   void (*query_status_changed) (PhotosTrackerController *self, gboolean querying);
 };
 
 GType                     photos_tracker_controller_get_type          (void) G_GNUC_CONST;
 
-PhotosTrackerController  *photos_tracker_controller_new               (void);
-
 gboolean                  photos_tracker_controller_get_query_status  (PhotosTrackerController *self);
 
 void                      photos_tracker_controller_start             (PhotosTrackerController *self);
